@@ -67,7 +67,7 @@
 #define INTERNETSECOND (864)
 #define INTERNETBEAT   (86400)
 
-#define N_GCONF_PREFS 6
+#define N_GCONF_PREFS 7
 
 #define NEVER_SENSITIVE "never_sensitive"
 
@@ -77,6 +77,7 @@ static const char* KEY_SHOW_DATE     = "show_date";
 static const char* KEY_GMT_TIME      = "gmt_time";
 static const char* KEY_CONFIG_TOOL   = "config_tool";
 static const char* KEY_CUSTOM_FORMAT = "custom_format";
+static const char* KEY_SHOW_WEEK     = "show_week_numbers";
 
 static const char *clock_config_tools [] = {
 	"system-config-date",
@@ -132,6 +133,7 @@ struct _ClockData {
 	gboolean     showseconds;
 	gboolean     showdate;
 	gboolean     gmt_time;
+	gboolean     showweek;
 
         char *config_tool;
 
@@ -680,7 +682,7 @@ handle_task_percent_complete_edited (ClockData           *cd,
         GtkTreeIter  iter;
         char        *uid;
         int          percent_complete;
-        char        *error = NULL;
+        char        *error = NULL, *text_copy;
 
         path       = gtk_tree_path_new_from_string (path_str);
         child_path = gtk_tree_model_filter_convert_path_to_child_path (cd->tasks_filter, path);
@@ -689,7 +691,10 @@ handle_task_percent_complete_edited (ClockData           *cd,
                             TASK_COLUMN_UID, &uid,
                             -1);
 
-        percent_complete = (int) g_strtod (text, &error);
+        text_copy = g_strdup (text);
+        text_copy = g_strdelimit (text_copy, "%", ' ');
+        text_copy = g_strstrip (text_copy);
+        percent_complete = (int) g_strtod (text_copy, &error);
         if (!error || !error [0]) {
                 gboolean task_completed;
 
@@ -703,6 +708,7 @@ handle_task_percent_complete_edited (ClockData           *cd,
         }
 
         g_free (uid);
+        g_free (text_copy);
         gtk_tree_path_free (path);
         gtk_tree_path_free (child_path);
 }
@@ -1309,7 +1315,8 @@ create_calendar (ClockData *cd,
                                    (gpointer *) &cd->calendar);
 
 	options = gtk_calendar_get_display_options (GTK_CALENDAR (cd->calendar));
-	options |= GTK_CALENDAR_SHOW_WEEK_NUMBERS;
+	if (cd->showweek)
+		options |= GTK_CALENDAR_SHOW_WEEK_NUMBERS;
 	gtk_calendar_set_display_options (GTK_CALENDAR (cd->calendar), options);
 
 	if (cd->gmt_time)
@@ -1930,6 +1937,41 @@ custom_format_changed (GConfClient  *client,
 		refresh_clock (clock);
 }
 
+static void
+show_week_changed (GConfClient  *client,
+		   guint         cnxn_id,
+		   GConfEntry   *entry,
+		   ClockData    *clock)
+{
+	GtkCalendarDisplayOptions options;
+	gboolean value;
+	
+	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+		return;
+
+	value = gconf_value_get_bool (entry->value);
+	
+	if (clock->showweek == (value != 0))
+		return;
+
+	clock->showweek = (value != 0);
+
+	if (clock->calendar_popup != NULL) {
+		options = gtk_calendar_get_display_options (GTK_CALENDAR (clock->calendar));
+
+		if (clock->showweek)
+			options |= GTK_CALENDAR_SHOW_WEEK_NUMBERS;
+		else
+			options &= ~(GTK_CALENDAR_SHOW_WEEK_NUMBERS);
+
+		gtk_calendar_set_display_options (GTK_CALENDAR (clock->calendar),
+						  options);
+
+                position_calendar_popup (clock, clock->calendar_popup,
+					 clock->toggle);
+	}
+}
+
 static void
 setup_gconf (ClockData *clock)
 {
@@ -1992,6 +2034,15 @@ setup_gconf (ClockData *clock)
 				clock, NULL, NULL);
 	g_free (key);
 	
+	key = panel_applet_gconf_get_full_key (PANEL_APPLET (clock->applet),
+					       KEY_SHOW_WEEK);
+	clock->listeners [6] =
+			gconf_client_notify_add (
+				client, key,
+				(GConfClientNotifyFunc) show_week_changed,
+				clock, NULL, NULL);
+	g_free (key);
+
 	g_object_unref (G_OBJECT (client));
 }
 
@@ -2078,6 +2129,7 @@ fill_clock_applet (PanelApplet *applet)
 	}
 
 	cd->gmt_time = panel_applet_gconf_get_bool (applet, KEY_GMT_TIME, NULL);
+	cd->showweek = panel_applet_gconf_get_bool (applet, KEY_SHOW_WEEK, NULL);
 	cd->config_tool = panel_applet_gconf_get_string (applet, KEY_CONFIG_TOOL, NULL);
 
 	cd->timeformat = NULL;
@@ -2106,7 +2158,6 @@ fill_clock_applet (PanelApplet *applet)
 			  G_CALLBACK (applet_change_pixel_size),
 			  cd);
 
-	/* FIXME: initial background, this needs some panel-applet voodoo */
 	g_signal_connect (G_OBJECT (cd->applet),
 			  "change_background",
 			  G_CALLBACK (applet_change_background),
@@ -2575,7 +2626,7 @@ clock_factory (PanelApplet *applet,
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_ClockApplet_Factory",
                              PANEL_TYPE_APPLET,
-                             "Clock Applet factory",
+                             "ClockApplet",
                              "0",
                              clock_factory,
                              NULL);
