@@ -78,8 +78,6 @@ static void display_help_dialog       (BonoboUIComponent *uic,
 static void display_about_dialog      (BonoboUIComponent *uic,
 				       PagerData         *pager,
 				       const gchar       *verbname);
-static void set_tooltip               (GtkWidget         *applet);
-
 
 static void
 pager_update (PagerData *pager)
@@ -179,6 +177,67 @@ applet_change_background (PanelApplet               *applet,
 	}
 }
 
+static gboolean
+applet_scroll (PanelApplet    *applet,
+               GdkEventScroll *event,
+               PagerData      *pager)
+{
+	WnckScreen *screen;
+	int         index;
+	int         n_workspaces;
+	int         n_columns;
+	int         in_last_row;
+	
+	if (event->type != GDK_SCROLL)
+		return FALSE;
+
+	screen         = applet_get_screen (GTK_WIDGET (applet));
+	index          = wnck_workspace_get_number (wnck_screen_get_active_workspace (screen));
+	n_workspaces   = wnck_screen_get_workspace_count (screen);
+	n_columns      = n_workspaces / pager->n_rows;
+	if (n_workspaces % pager->n_rows != 0)
+		n_columns++;
+	in_last_row    = n_workspaces % n_columns;
+	
+	switch (event->direction) {
+	case GDK_SCROLL_DOWN:
+		if (index + n_columns < n_workspaces)
+			index += n_columns;
+		else if ((index < n_workspaces - 1
+			  && index + in_last_row != n_workspaces - 1) ||
+			 (index == n_workspaces - 1
+			  && in_last_row != 0))
+			index = (index % n_columns) + 1;
+		break;
+		
+	case GDK_SCROLL_RIGHT:
+		if (index < n_workspaces)
+			index++;
+		break;
+		
+	case GDK_SCROLL_UP:
+		if (index - n_columns >= 0)
+			index -= n_columns;
+		else if (index > 0)
+			index = ((pager->n_rows - 1) * n_columns) + (index % n_columns) - 1;
+			if (index >= n_workspaces)
+				index -= n_columns;
+		break;
+
+	case GDK_SCROLL_LEFT:
+		if (index > 0)
+			index--;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	wnck_workspace_activate (wnck_screen_get_workspace (screen, index));
+	
+	return TRUE;
+}
+
 static void 
 response_cb (GtkWidget *widget,
 	     int        id,
@@ -445,10 +504,8 @@ workspace_switcher_applet_fill (PanelApplet *applet)
 
 	pager_update (pager);
 	
-	gtk_widget_show (pager->pager);
 	gtk_container_add (GTK_CONTAINER (pager->applet), pager->pager);
-
-	set_tooltip (GTK_WIDGET (pager->pager));
+	gtk_widget_show (pager->pager);
 
 	gtk_widget_show (pager->applet);
 
@@ -460,6 +517,10 @@ workspace_switcher_applet_fill (PanelApplet *applet)
 			  "change_orient",
 			  G_CALLBACK (applet_change_orient),
 			  pager);
+	g_signal_connect (G_OBJECT (pager->applet),
+			  "scroll-event",
+			  G_CALLBACK (applet_scroll),
+			  pager);
 	g_signal_connect (G_OBJECT (pager->applet),
 			  "change_background",
 			  G_CALLBACK (applet_change_background),
@@ -939,20 +1000,3 @@ display_properties_dialog (BonoboUIComponent *uic,
 			       gtk_widget_get_screen (pager->applet));
 	gtk_window_present (GTK_WINDOW (pager->properties_dialog));
 }
-
-/*
- *Function to set tooltip
- */
-static void
-set_tooltip (GtkWidget *applet)
-{
-	GtkTooltips *tooltips;
-
-	tooltips = gtk_tooltips_new ();
-	g_object_ref (tooltips);
-	gtk_object_sink (GTK_OBJECT (tooltips));
-	g_object_set_data_full (G_OBJECT (applet), "tooltips", tooltips,
-                                (GDestroyNotify) g_object_unref);
-        gtk_tooltips_set_tip (tooltips, applet,
-                              _("Workspace Switcher"), NULL);
-}
