@@ -17,6 +17,7 @@
 #include "drawer.h"
 #include "launcher.h"
 #include "menu.h"
+#include "panel-addto.h"
 #include "panel-gconf.h"
 #include "panel-config-global.h"
 #include "panel-applet-frame.h"
@@ -33,9 +34,6 @@
 
 #define SMALL_ICON_SIZE 20
 
-static GtkWidget *panel_applet_create_menu (AppletInfo  *info,
-					    PanelWidget *panel_widget);
-
 static GSList *registered_applets = NULL;
 static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
@@ -90,15 +88,14 @@ panel_applet_toggle_locked (AppletInfo *info)
 }
 
 static void
-panel_applet_lock (GtkWidget  *menuitem,
-		   AppletInfo *info)
+panel_applet_lock (GtkCheckMenuItem *menuitem,
+		   AppletInfo       *info)
 {
 	gboolean locked;
 
 	locked = panel_applet_toggle_locked (info);
 
-	gtk_label_set_label (GTK_LABEL (GTK_BIN (menuitem)->child),
-			     locked ? _("Un_lock") : _("_Lock"));
+	gtk_check_menu_item_set_active (menuitem, locked);
 
 	if (info->move_item)
 		gtk_widget_set_sensitive (info->move_item, !locked);
@@ -156,8 +153,47 @@ panel_applet_recreate_menu (AppletInfo	*info)
 	}
 
 	gtk_widget_unref (info->menu);
-	info->menu = panel_applet_create_menu (info,
-					       PANEL_WIDGET (info->widget->parent));
+	info->menu = panel_applet_create_menu (info);
+}
+
+static void
+panel_applet_locked_change_notify (GConfClient *client,
+				   guint        cnxn_id,
+				   GConfEntry  *entry,
+				   GtkWidget   *applet)
+{
+	GConfValue  *value;
+	gboolean     locked;
+	gboolean     applet_locked;
+	AppletInfo  *info;
+	PanelWidget *panel_widget;
+
+	g_assert (applet != NULL);
+
+	info = (AppletInfo  *) g_object_get_data (G_OBJECT (applet),
+						  "applet_info");
+	if (info == NULL)
+		return;
+
+	value = gconf_entry_get_value (entry);
+	if (value->type != GCONF_VALUE_BOOL)
+		return;
+
+	locked = gconf_value_get_bool (value);
+
+	panel_widget = PANEL_WIDGET (info->widget->parent);
+	applet_locked = panel_widget_get_applet_locked (panel_widget,
+							info->widget);
+
+	if ((locked && applet_locked) || !(locked || applet_locked))
+		return;
+
+	panel_applet_toggle_locked (info);
+
+	if (info->type == PANEL_OBJECT_BONOBO)
+		panel_applet_frame_sync_menu_state (PANEL_APPLET_FRAME (info->widget));
+	else
+		panel_applet_recreate_menu (info);
 }
 
 static void
@@ -193,12 +229,18 @@ applet_callback_callback (GtkWidget      *widget,
 
 	switch (menu->info->type) {
 	case PANEL_OBJECT_LAUNCHER:
-		if (!strcmp (menu->name, "properties"))
-			launcher_properties (
-				menu->info->data, screen);
+		if (!strcmp (menu->name, "launch"))
+			launcher_launch (menu->info->data, widget);
+		else if (!strcmp (menu->name, "properties"))
+			launcher_properties (menu->info->data, screen);
 		break;
 	case PANEL_OBJECT_DRAWER: 
-		if (strcmp (menu->name, "properties")==0) {
+		if (strcmp (menu->name, "add") == 0) {
+			Drawer *drawer = menu->info->data;
+
+			panel_addto_present (GTK_MENU_ITEM (widget),
+					     panel_toplevel_get_panel_widget (drawer->toplevel));
+		} else if (strcmp (menu->name, "properties") == 0) {
 			Drawer *drawer = menu->info->data;
 
 			panel_properties_dialog_present (drawer->toplevel);
@@ -233,7 +275,20 @@ applet_callback_callback (GtkWidget      *widget,
 }
 
 static void
-applet_menu_deactivate (GtkWidget *w, AppletInfo *info)
+applet_menu_show (GtkWidget *w,
+		  AppletInfo *info)
+{
+	PanelWidget *panel_widget;
+
+	panel_widget = PANEL_WIDGET (info->widget->parent);
+
+	panel_toplevel_push_autohide_disabler (panel_widget->toplevel);
+}
+
+
+static void
+applet_menu_deactivate (GtkWidget *w,
+			AppletInfo *info)
 {
 	PanelWidget *panel_widget;
 
@@ -397,21 +452,25 @@ add_to_submenus (AppletInfo *info,
 	g_free(n);
 }
 
-static GtkWidget *
-panel_applet_create_menu (AppletInfo  *info,
-			  PanelWidget *panel_widget)
+GtkWidget *
+panel_applet_create_menu (AppletInfo *info)
 {
-	GtkWidget *menu;
-	GtkWidget *menuitem;
-	GList     *l;
-	gboolean   added_anything = FALSE;
+	GtkWidget   *menu;
+	GtkWidget   *menuitem;
+	GList       *l;
+	PanelWidget *panel_widget;
+	gboolean     added_anything = FALSE;
+
+	panel_widget = PANEL_WIDGET (info->widget->parent);
 
 	menu = g_object_ref (panel_create_menu ());
 	gtk_object_sink (GTK_OBJECT (menu));
 
-	/* connect the deactivate signal, so that we can "re-allow" 
-	 * autohide when the menu is deactivated.
+	/* connect the show & deactivate signal, so that we can "disallow" and
+	 * "re-allow" autohide when the menu is shown/deactivated.
 	 */
+	g_signal_connect (menu, "show",
+			  G_CALLBACK (applet_menu_show), info);
 	g_signal_connect (menu, "deactivate",
 			  G_CALLBACK (applet_menu_deactivate), info);
 
@@ -455,17 +514,17 @@ panel_applet_create_menu (AppletInfo  *info,
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 		gtk_widget_set_sensitive (menuitem, removable);
 
-		menuitem = gtk_image_menu_item_new ();
-		image = gtk_image_new ();
-		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, locked ? _("Un_lock") : _("_Lock"), FALSE);
-		g_signal_connect (menuitem, "activate",
+		menuitem = gtk_check_menu_item_new ();
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, NULL, _("_Lock To Panel"), FALSE);
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
+						locked);
+		g_signal_connect (menuitem, "toggled",
 				  G_CALLBACK (panel_applet_lock), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 		gtk_widget_set_sensitive (menuitem, lockable);
 		
 		menuitem = gtk_image_menu_item_new ();
-		image = gtk_image_new ();
-		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, _("_Move"), FALSE);
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, NULL, _("_Move"), FALSE);
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (move_applet_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
@@ -564,13 +623,11 @@ applet_show_menu (AppletInfo     *info,
 	panel_widget = PANEL_WIDGET (info->widget->parent);
 
 	if (info->menu == NULL)
-		info->menu = panel_applet_create_menu (info, panel_widget);
+		info->menu = panel_applet_create_menu (info);
 
 	if (info->menu == NULL)
 		return;
 
-	panel_toplevel_push_autohide_disabler (panel_widget->toplevel);
-
 	panel_applet_menu_set_recurse (GTK_MENU (info->menu),
 				       "menu_panel",
 				       info->widget->parent);
@@ -659,8 +716,9 @@ panel_applet_destroy (GtkWidget  *widget,
 		}
 	}
 
-	panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menu),
-				      info);
+	if (info->type != PANEL_OBJECT_BONOBO)
+		panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menu),
+					      info);
 
 	if (info->menu)
 		g_object_unref (info->menu);
@@ -1070,6 +1128,7 @@ panel_applet_register (GtkWidget       *applet,
 		       const char      *id)
 {
 	AppletInfo *info;
+	const char *key;
 	
 	g_return_val_if_fail (applet != NULL && panel != NULL, NULL);
 
@@ -1091,8 +1150,16 @@ panel_applet_register (GtkWidget       *applet,
 
 	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
-	panel_lockdown_notify_add (G_CALLBACK (panel_applet_recreate_menu),
-				   info);
+	if (type != PANEL_OBJECT_BONOBO)
+		panel_lockdown_notify_add (G_CALLBACK (panel_applet_recreate_menu),
+					   info);
+
+	key = panel_gconf_full_key ((type == PANEL_OBJECT_BONOBO) ?
+				     PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS,
+				    id, "locked");
+	panel_gconf_notify_add_while_alive (key,
+					    (GConfClientNotifyFunc) panel_applet_locked_change_notify,
+					    G_OBJECT (applet));
 
 	if (type == PANEL_OBJECT_DRAWER) {
 		Drawer *drawer = data;
