@@ -59,6 +59,7 @@ typedef struct {
 
 	GSList       *applet_list;
 	GSList       *application_list;
+	GSList       *settings_list;
 
 	guint         name_notify;
 
@@ -192,7 +193,7 @@ panel_addto_applet_info_sort_func (PanelAddtoItemInfo *a,
 }
 
 static GSList *
-panel_addto_append_internal_applets (GSList *list)
+panel_addto_prepend_internal_applets (GSList *list)
 {
 	static gboolean translated = FALSE;
 	int             i;
@@ -203,7 +204,7 @@ panel_addto_append_internal_applets (GSList *list)
 			internal_addto_items [i].description = _(internal_addto_items [i].description);
 		}
 
-                list = g_slist_append (list, &internal_addto_items [i]);
+                list = g_slist_prepend (list, &internal_addto_items [i]);
         }
 
 	translated = TRUE;
@@ -224,7 +225,7 @@ panel_addto_append_internal_applets (GSList *list)
 		info->iid         = (char *) panel_action_get_drag_id (i);
 		info->static_data = TRUE;
 
-                list = g_slist_append (list, info);
+                list = g_slist_prepend (list, info);
 	}
 
         return list;
@@ -239,7 +240,7 @@ panel_addto_make_text (const char *name,
 
 	real_name = name ? name : _("(empty)");
 
-	if (desc != NULL && desc[0] != '\0') {
+	if (!string_empty (desc)) {
 		result = g_markup_printf_escaped ("<span size=\"larger\" weight=\"bold\">%s</span>\n%s",
 						  real_name, desc);
 	} else {
@@ -424,7 +425,7 @@ panel_addto_query_applets (GSList *list)
 		applet->iid = g_strdup (info->iid);
 		applet->static_data = FALSE;
 
-		list = g_slist_append (list, applet);
+		list = g_slist_prepend (list, applet);
 	}
 
 	g_slist_free (langs_gslist);
@@ -503,7 +504,7 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 
 	if (panel_profile_id_lists_are_writable ()) {
 		dialog->applet_list = panel_addto_query_applets (dialog->applet_list);
-		dialog->applet_list = panel_addto_append_internal_applets (dialog->applet_list);
+		dialog->applet_list = panel_addto_prepend_internal_applets (dialog->applet_list);
 	}
 
 	dialog->applet_list = g_slist_sort (dialog->applet_list,
@@ -557,7 +558,7 @@ panel_addto_make_application_list (GSList            **parent_list,
 		 * So the iid is built when we select the row.
 		 */
 
-		*parent_list = g_slist_append (*parent_list, data);
+		*parent_list = g_slist_prepend (*parent_list, data);
 
 		panel_addto_make_application_list (&data->children, subdir);
 
@@ -579,11 +580,13 @@ panel_addto_make_application_list (GSList            **parent_list,
 		data->item_info.launcher_path = g_strdup (menu_tree_entry_get_desktop_file_path (entry));
 		data->item_info.static_data = FALSE;
 
-		*parent_list = g_slist_append (*parent_list, data);
+		*parent_list = g_slist_prepend (*parent_list, data);
 
 		menu_tree_entry_unref (entry);
 	}
 	g_slist_free (entries);
+
+	*parent_list = g_slist_reverse (*parent_list);
 }
 
 static void
@@ -648,6 +651,29 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 
 	menu_tree_unref (tree);
 
+	tree = menu_tree_lookup ("settings.menu");
+
+	if ((root = menu_tree_get_root_directory (tree))) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_append (store, &iter, NULL);
+		gtk_tree_store_set (store, &iter,
+				    COLUMN_ICON, NULL,
+				    COLUMN_TEXT, NULL,
+				    COLUMN_DATA, NULL,
+				    COLUMN_SEARCH, NULL,
+				    -1);
+
+		panel_addto_make_application_list (&dialog->settings_list,
+						   root);
+		panel_addto_populate_application_model (store, NULL,
+							dialog->settings_list);
+
+		menu_tree_directory_unref (root);
+	}
+
+	menu_tree_unref (tree);
+
 	return GTK_TREE_MODEL (store);
 }
 
@@ -866,6 +892,7 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 	g_slist_free (dialog->applet_list);
 
 	panel_addto_dialog_free_application_list (dialog->application_list);
+	panel_addto_dialog_free_application_list (dialog->settings_list);
 
 	if (dialog->menu_tree)
 		menu_tree_unref (dialog->menu_tree);
@@ -879,28 +906,29 @@ panel_addto_name_change (PanelAddtoDialog *dialog,
 			 const char       *name)
 {
 	char *title;
-	char *markup_title;
+	char *label;
 
-	if (name) {
-		title = g_strdup_printf (_("Add to %s"), name);
+	label = NULL;
+
+	if (!string_empty (name))
+		label = g_strdup_printf (_("Select an _item to add to \"%s\":"),
+					 name);
+
+	if (panel_toplevel_get_is_attached (dialog->panel_widget->toplevel)) {
+		title = g_strdup_printf (_("Add to Drawer"));
+		if (label == NULL)
+			label = g_strdup (_("Select an _item to add to the drawer:"));
 	} else {
-		title = g_strdup_printf (_("Add to the panel"));
+		title = g_strdup_printf (_("Add to Panel"));
+		if (label == NULL)
+			label = g_strdup (_("Select an _item to add to the panel:"));
 	}
+
 	gtk_window_set_title (GTK_WINDOW (dialog->addto_dialog), title);
 	g_free (title);
 
-	if (name) {
-		title = g_strdup_printf (_("Select an _item to add to %s:"),
-					 name);
-	} else {
-		title = g_strdup (_("Select an _item to add to the panel:"));
-	}
-	markup_title = g_strdup_printf ("<span weight=\"bold\">%s</span>",
-					title);
-	g_free (title);
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (dialog->label),
-					    markup_title);
-	g_free (markup_title);
+	gtk_label_set_text_with_mnemonic (GTK_LABEL (dialog->label), label);
+	g_free (label);
 }
 
 static void
@@ -953,14 +981,12 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 	if (data->type == PANEL_ADDTO_LAUNCHER_MENU) {
 		gtk_button_set_label (GTK_BUTTON (dialog->add_button),
 				      GTK_STOCK_GO_FORWARD);
-		gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
-					  TRUE);
 	} else {
 		gtk_button_set_label (GTK_BUTTON (dialog->add_button),
 				      GTK_STOCK_ADD);
-		gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
-					  TRUE);
 	}
+	gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
+				  TRUE);
 
 	/* only allow dragging applets if we can add applets */
 	if (panel_profile_id_lists_are_writable ()) {
@@ -1053,12 +1079,12 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	dialog->addto_dialog = gtk_dialog_new ();
 	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
 			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
-	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-			       GTK_STOCK_CANCEL,
-			       GTK_RESPONSE_CANCEL);
 	dialog->back_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
 						     GTK_STOCK_GO_BACK,
 						     PANEL_ADDTO_RESPONSE_BACK);
+	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
+			       GTK_STOCK_CANCEL,
+			       GTK_RESPONSE_CANCEL);
 	dialog->add_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
 						     GTK_STOCK_ADD,
 						     PANEL_ADDTO_RESPONSE_ADD);
@@ -1068,15 +1094,15 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->addto_dialog),
 					 PANEL_ADDTO_RESPONSE_ADD);
 
-	gtk_container_set_border_width (GTK_CONTAINER (dialog->addto_dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog->addto_dialog)->vbox), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog->addto_dialog), 6);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog->addto_dialog)->vbox), 12);
 	g_signal_connect (G_OBJECT (dialog->addto_dialog), "response",
 			  G_CALLBACK (panel_addto_dialog_response), dialog);
 	g_signal_connect (dialog->addto_dialog, "destroy",
 			  G_CALLBACK (panel_addto_dialog_destroy), dialog);
 
 	vbox = gtk_vbox_new (FALSE, 12);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog->addto_dialog)->vbox),
 			   vbox);
 
