@@ -89,7 +89,7 @@ recent_documents_activate_cb (EggRecentViewGtk *view, EggRecentItem *item,
 	if (show_uri (uri, mime_type, screen, &error) != TRUE) {
 		if (error != NULL) {
 			panel_error_dialog (screen,
-					    "cannot_open_recent_doc",
+					    "cannot_open_recent_doc", TRUE,
 					    _("Cannot open recently used document"),
 					    "%s",
 					    error->message);
@@ -97,6 +97,7 @@ recent_documents_activate_cb (EggRecentViewGtk *view, EggRecentItem *item,
 		} else
 			panel_error_dialog (screen,
 					    "cannot_open_recent_doc_unknown",
+					    TRUE,
 					    _("Cannot open recently used document"),
 					    _("An unknown error occurred while trying to open %s"),
 					    uri_utf8);
@@ -214,11 +215,12 @@ panel_recent_append_documents_menu (GtkWidget *top_menu)
 	EggRecentViewGtk *view;
 
 	menu_item = gtk_image_menu_item_new ();
-	setup_stock_menu_item (menu_item,
-			       panel_menu_icon_get_size (),
-			       GTK_STOCK_OPEN,
-			       _("Recent Documents"),
-			       TRUE);
+	setup_menu_item_with_icon (menu_item,
+				   panel_menu_icon_get_size (),
+				   NULL,
+				   GTK_STOCK_OPEN,
+				   _("Recent Documents"),
+				   TRUE);
 	menu = panel_create_menu ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
 
@@ -239,15 +241,15 @@ panel_recent_append_documents_menu (GtkWidget *top_menu)
 	egg_recent_view_gtk_set_tooltip_func (view,
 					      recent_documents_tooltip_func,
 					      NULL);
-	menu_item = gtk_separator_menu_item_new ();
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	add_menu_separator (menu);
 
 	menu_item = gtk_image_menu_item_new ();
-	setup_stock_menu_item (menu_item,
-			       panel_menu_icon_get_size (),
-			       GTK_STOCK_CLEAR,
-			       _("Clear Recent Documents"),
-			       TRUE);
+	setup_menu_item_with_icon (menu_item,
+				   panel_menu_icon_get_size (),
+				   NULL,
+				   GTK_STOCK_CLEAR,
+				   _("Clear Recent Documents"),
+				   TRUE);
 	gtk_tooltips_set_tip (panel_tooltips,
 			      menu_item,
 			      _("Clear all items from the recent documents list"),
