@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2003,2004 Rob Adams
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -286,20 +287,24 @@ panel_struts_set_window_hint (PanelToplevel *toplevel)
 
 	switch (strut->orientation) {
 	case PANEL_ORIENTATION_TOP:
-		if (topmost && monitor_y > 0)
+		if (monitor_y > 0)
 			strut_size += monitor_y;
+		if (!topmost) strut_size = 0;
 		break;
 	case PANEL_ORIENTATION_BOTTOM:
-		if (bottommost && monitor_y + monitor_height < screen_height)
+		if (monitor_y + monitor_height < screen_height)
 			strut_size += screen_height - (monitor_y + monitor_height);
+		if (!bottommost) strut_size = 0;
 		break;
 	case PANEL_ORIENTATION_LEFT:
 		if (leftmost && monitor_x > 0)
 			strut_size += monitor_x;
+		if (!leftmost) strut_size = 0;
 		break;
 	case PANEL_ORIENTATION_RIGHT:
-		if (rightmost && monitor_x + monitor_width < screen_width)
+		if (monitor_x + monitor_width < screen_width)
 			strut_size += screen_width - (monitor_x + monitor_width);
+		if (!rightmost) strut_size = 0;
 		break;
 	default:
 		g_assert_not_reached ();
@@ -331,17 +336,32 @@ orientation_to_order (PanelOrientation orientation)
         }
 }
 
+static inline int
+get_toplevel_depth (PanelToplevel *toplevel)
+{
+	int depth = 0;
+
+	while ((toplevel = panel_toplevel_get_attach_toplevel (toplevel)))
+		depth++;
+
+	return depth;
+}
+
 /* Sort in order of
  *   1) screen
  *   2) monitor
- *   3) top, bottom, left, right
- *   4) strut_start ascending
- *   5) strut_end descending
+ *   3) depth (for drawers)
+ *   4) top, bottom, left, right
+ *   5) strut_start ascending
+ *   6) strut_end descending
  */
 static int
 panel_struts_compare (const PanelStrut *s1,
 		      const PanelStrut *s2)
 {
+	int s1_depth;
+	int s2_depth;
+
 	if (s1->screen != s2->screen)
 		return gdk_screen_get_number (s1->screen) -
 			gdk_screen_get_number (s2->screen);
@@ -349,6 +369,11 @@ panel_struts_compare (const PanelStrut *s1,
 	if (s1->monitor != s2->monitor)
 		return s1->monitor - s2->monitor;
 
+	s1_depth = get_toplevel_depth (s1->toplevel);
+	s2_depth = get_toplevel_depth (s2->toplevel);
+	if (s1_depth != s2_depth)
+		return s2_depth - s1_depth;
+
         if (s1->orientation != s2->orientation)
                 return orientation_to_order (s1->orientation) -
 			orientation_to_order (s2->orientation);
