@@ -59,6 +59,7 @@ panel_show_help (GdkScreen  *screen,
 		panel_error_dialog (
 			screen,
 			"cannot_show_help",
+			TRUE,
 			_("Cannot display help document"),
 			"%s",
 			error != NULL ? error->message : "");
@@ -202,7 +203,8 @@ static GtkWidget *
 panel_dialog (GdkScreen  *screen,
 	      int         type,
 	      const char *class,
-	      const char *str)
+	      const char *str,
+	      gboolean    auto_destroy)
 {
 	GtkWidget *dialog;
 
@@ -221,11 +223,10 @@ panel_dialog (GdkScreen  *screen,
 
 	gtk_widget_show_all (dialog);
 
-	/* FIXME: this is ugly and makes it bad to run gtk_dialog_run
-	 * after this function */
-	g_signal_connect_swapped (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (dialog));
+	if (auto_destroy)
+		g_signal_connect_swapped (G_OBJECT (dialog), "response",
+					  G_CALLBACK (gtk_widget_destroy),
+					  G_OBJECT (dialog));
 
 	return dialog;
 }
@@ -233,6 +234,7 @@ panel_dialog (GdkScreen  *screen,
 GtkWidget *
 panel_error_dialog (GdkScreen  *screen,
 		    const char *class,
+		    gboolean    auto_destroy,
 		    const char *primary_format,
 		    const char *secondary_format,
 		    ...)
@@ -265,7 +267,7 @@ panel_error_dialog (GdkScreen  *screen,
 			g_free (format);
 	}
 
-	w = panel_dialog (screen, GTK_MESSAGE_ERROR, class, s);
+	w = panel_dialog (screen, GTK_MESSAGE_ERROR, class, s, auto_destroy);
 	g_free (s);
 	return w;
 }
@@ -536,6 +538,7 @@ panel_lock_screen (GdkScreen *screen)
 	if (!gdk_spawn_command_line_on_screen (screen, XSCREENSAVER_LOCK_COMMAND, &error)) {
 		panel_error_dialog (screen,
 				    "cannot_exec_xscreensaver",
+				    TRUE,
 				    _("Cannot execute '%s'"),
 				    "%s",
 				    XSCREENSAVER_LOCK_COMMAND,
@@ -621,3 +624,39 @@ panel_make_unique_uri (const char *dir,
 
 #undef NUM_OF_WORDS
 }
+
+static char *
+lookup_in_data_dir (const char *basename,
+                    const char *data_dir)
+{
+	char *path;
+
+	path = g_build_filename (data_dir, basename, NULL);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
+		g_free (path);
+		return NULL;
+	}
+
+	return path;
+}
+
+char *
+panel_lookup_in_data_dirs (const char *basename)
+{
+	const char * const *system_data_dirs;
+	const char          *user_data_dir;
+	char                *retval;
+	int                  i;
+
+	user_data_dir    = g_get_user_data_dir ();
+	system_data_dirs = g_get_system_data_dirs ();
+
+	if ((retval = lookup_in_data_dir (basename, user_data_dir)))
+		return retval;
+
+	for (i = 0; system_data_dirs[i]; i++)
+		if ((retval = lookup_in_data_dir (basename, system_data_dirs[i])))
+			return retval;
+
+	return NULL;
+}
