@@ -2,13 +2,8 @@
 #define PANEL_UTIL_H
 
 #include <gtk/gtk.h>
-#include <gdk/gdk.h>
-
 #include <libgnome/gnome-desktop-item.h>
 
-#include "panel-widget.h"
-#include "panel-config-global.h"
-
 G_BEGIN_DECLS
 
 typedef void (*UpdateFunction) (gpointer);
@@ -41,11 +36,10 @@ GList *panel_g_list_swap_prev     (GList        *list,
 GList *panel_g_list_resort_item   (GList        *list,
 				   gpointer      data,
 				   GCompareFunc  func);
-void   panel_g_slist_deep_free    (GSList       *list);
-
 
 GtkWidget      *panel_error_dialog      (GdkScreen  *screen,
 					 const char *class,
+					 gboolean    auto_destroy,
 					 const char *primary_format,
 					 const char *secondary_format,
 					 ...);
@@ -62,26 +56,13 @@ gboolean	panel_uri_exists	(const char *uri);
 
 void            panel_lock_screen       (GdkScreen *screen);
 
-
-/* GnomeVFS reading utils */
-
-typedef struct _ReadBuf ReadBuf;
-
-/* Note, does not include the trailing \n */
-char *		readbuf_gets		(char *buf, 
-					 gsize bufsize,
-					 ReadBuf *rb);
-ReadBuf *	readbuf_open		(const char *uri);
-/* unused for now */
-gboolean	readbuf_rewind		(ReadBuf *rb);
-void		readbuf_close		(ReadBuf *rb);
-
 GdkPixbuf *	missing_pixbuf		(int size);
 
 char *panel_make_full_path   (const char *dir,
 			      const char *filename);
 char *panel_make_unique_uri (const char *dir,
 			     const char *suffix);
+char *panel_lookup_in_data_dirs (const char *basename);
 
 G_END_DECLS
 
